@@ -1,4 +1,3 @@
-import os
 import sys
 import json
 import glob
@@ -8,6 +7,7 @@ import logomaker
 import subprocess
 import numpy as np
 import pandas as pd
+from os import path
 from Bio import SeqIO, AlignIO
 from itertools import product
 import matplotlib
@@ -21,8 +21,6 @@ import pathlib
 abs_path = str(pathlib.Path(__file__).parent.resolve())
 
 class HMMScanner():
-    def __init__(self):
-        pass
 
     def get_UP_seq(self, UP_id):
         URL = "https://rest.uniprot.org/uniprotkb/{}.fasta".format(UP_id)
@@ -112,51 +110,68 @@ class HMMScanner():
                     seqs_dict[seqid] = updated_seq
 
             return seqs_dict
-        
-
 
 class RNAScoring():
     def __init__(self):
-        self.res_list = ['R', 'K', 'H', 'D', 'E', 'Q', 'N', 'S', 'T', 'Y', 'F',
-                    'W', 'M', 'L', 'I', 'V', 'A', 'G', 'P', 'C'
-                    ]
-        self.nt_list = ["A", "G", "C", "U"]
+        self.scores_dict = {}
 
         # Protein bias
-        fasta_rna_aln = abs_path + \
-                "/alignment_data/RNAs_aligned_cluster_0_nt_full_refined.fasta"
-        rna_seqs = list(SeqIO.parse(open(fasta_rna_aln), 'fasta'))
-        self.rna_seqs_dict_full = {i.id: str(i.seq) for i in rna_seqs}
-        self.bias_dict = Counter(
-            [i.id.split("_")[0] + "_" + i.id.split("_")[1] for i in rna_seqs])
-
-        self.fasta_prot_aln = abs_path + \
-          "/alignment_data/rrm_bound_domains_aligned_processed_squeezed.fasta"
-          
-        prot_seqs = list(SeqIO.parse(open(self.fasta_prot_aln), 'fasta'))
-        self.prot_seqs_dict_full = {"_".join(i.id.split("_")[:2]): str(i.seq) for i in prot_seqs}
-
+        self.fasta_rna_aln = path.join(abs_path,
+                "alignment_data/RNAs_aligned_cluster_0_nt_full_refined.fasta")
+        self.fasta_prot_aln = path.join(abs_path,
+          "alignment_data/rrm_bound_domains_aligned_processed_squeezed.fasta")
         # Input data
-        master_aln = abs_path + \
-            '/alignment_data/RRM_master_interaction_info_with_nt_unbiased_cluster_0.json'
-        self.aln_data = json.load(open(master_aln))
-        rna_aln = abs_path + "/alignment_data/neighbours_rna_cluster_0.json"
-        self.rna_data = json.load(open(rna_aln))
-        self.entries_list = list(self.rna_seqs_dict_full.keys())
+        self.master_aln = path.join(abs_path,
+            'alignment_data/RRM_master_interaction_info_with_nt_unbiased_cluster_0.json')
 
-        self.hmm_to_master_pos = {
-            29: 77, 31: 79, 33: 81, 34: 82, 37: 90, 58: 122, 60: 124, 62: 126,
-            66: 135, 68: 137, 70: 139, 72: 141, 94: 166, 97: 169, 99: 171,
-            101: 173, 102: 174, 103: 175, 104: 176, 105: 177
-        }
+        self.rna_aln = path.join(abs_path, "alignment_data/neighbours_rna_cluster_0.json")
 
+        # self.hmm_to_master_pos = {
+        #     29: 77, 31: 79, 33: 81, 34: 82, 37: 90, 58: 122, 60: 124, 62: 126,
+        #     66: 135, 68: 137, 70: 139, 72: 141, 94: 166, 97: 169, 99: 171,
+        #     101: 173, 102: 174, 103: 175, 104: 176, 105: 177
+        # }
 
-    def load_scores(self, scores_folder=abs_path + "/alignment_data/precalculated_scores/*.pkl"):
+    @property
+    def rna_seqs(self):
+        return list(SeqIO.parse(open(self.fasta_rna_aln), 'fasta'))
+
+    @property
+    def rna_seqs_dict_full(self):
+        return {i.id: str(i.seq) for i in self.rna_seqs}
+
+    @property
+    def bias_dict(self):
+        return Counter([i.id.split("_")[0] + "_" + i.id.split("_")[1] for i in self.rna_seqs])
+
+    @property
+    def prot_seqs(self):
+        return list(SeqIO.parse(open(self.fasta_prot_aln), 'fasta'))
+
+    @property
+    def prot_seqs_dict_full(self):
+        return {"_".join(i.id.split("_")[:2]): str(i.seq) for i in self.prot_seqs}
+
+    @property
+    def aln_data(self):
+        return json.load(open(self.master_aln))
+
+    @property
+    def rna_data(self):
+        return json.load(open(self.rna_aln))
+
+    @property
+    def entries_list(self):
+        return list(self.rna_seqs_dict_full.keys())
+
+    def load_scores(self, scores_folder=path.join(abs_path,"alignment_data/precalculated_scores/*.pkl")):
         self.all_df_dict = {}
+
         for file in glob.glob(scores_folder):
             df = pd.read_pickle(file)
             res_num = int(file.split("/")[-1].split(".")[0].split("_")[0])
             nuc_num = int(file.split("/")[-1].split(".")[0].split("_")[1])
+
             self.all_df_dict[(res_num, nuc_num)] = df
 
         return self.all_df_dict
@@ -181,15 +196,17 @@ class RNAScoring():
 
         return self.scores_dict
 
-    def plot_rna_kde(self, seq_id, rna_seq, scores_dict, window_size):
-        pos_file = open(abs_path +
-            "/alignment_data/leave-one-out_positive_scores_avg_perc_20.txt",
-            "r")
-        pos_scores = [float(i.strip()) for i in pos_file.readlines()]
-        neg_file = open(abs_path +
-            "/alignment_data/leave-one-out_negative_scores_avg_perc_20.txt",
-            "r")
-        neg_scores = [float(i.strip()) for i in neg_file.readlines()]
+    def plot_rna_kde_to_file(self, rna_seq, window_size, plot_path, dpi=300):
+        self.plot_rna_kde(rna_seq=rna_seq, scores_dict=self.scores_dict, window_size=window_size)
+        plt.savefig(plot_path, dpi=dpi)
+
+    def plot_rna_kde(self, rna_seq, scores_dict, window_size):
+        with open(path.join(abs_path, "alignment_data/leave-one-out_positive_scores_avg_perc_20.txt"), "r") as pos_file:
+            pos_scores = [float(i.strip()) for i in pos_file.readlines()]
+
+        with open(path.join(abs_path, "alignment_data/leave-one-out_negative_scores_avg_perc_20.txt"), "r") as neg_file:
+            neg_scores = [float(i.strip()) for i in neg_file.readlines()]
+
         pos_kde = KernelDensity(kernel='gaussian', bandwidth=0.5).fit(
             np.array(pos_scores).reshape(-1, 1))
         neg_kde = KernelDensity(kernel='gaussian', bandwidth=0.5).fit(
@@ -245,8 +262,7 @@ class RNAScoring():
 
     def find_best_rna(self, rrm_seq):
         # To find the best 5-mer RNA for a rrm_seq
-        nt_list = [["A", "C", "G", "U"]]
-        mer5_list = list(product(*nt_list * 5))
+        mer5_list = list(product(*LogoGenerator.NT_LIST * 5))
         # Calculate the scores for all the 3-mer RNA fragments
         mer5_scores = {}
         for rna_seq in mer5_list:
@@ -284,15 +300,14 @@ class LogoGenerator:
 
     """
 
-    def __init__(self):
-        self.nt_list = [["A", "C", "G", "U"]]
+    NT_LIST = [["A", "C", "G", "U"]]
 
     def generate_PPM(self, rna_scores, n_mers, ws=5):
         # PPM = Position Probability Matrix
         ppm = pd.DataFrame(columns=['A', 'C', 'G', "U"])
         for i in range(ws):
             nt_obs = [mer[i] for mer in list(rna_scores.keys())[:n_mers]]
-            freqs = [nt_obs.count(nt) / n_mers for nt in self.nt_list[0]]
+            freqs = [nt_obs.count(nt) / n_mers for nt in LogoGenerator.NT_LIST[0]]
             # Create a new DataFrame with the freqs values
             new_row = pd.DataFrame({
                 'A': [freqs[0]],
@@ -330,27 +345,33 @@ class LogoGenerator:
             bits_df = pd.concat([bits_df, new_row], ignore_index=True)
         return bits_df
 
-    def retrieve_bits_stats(self, bits_df, json_path=False):
-        max_bits_sum = sum([max(bits_df.iloc[ind]) for ind in bits_df.index])
-        total_bits_sum = sum([sum(bits_df.iloc[ind]) for ind in bits_df.index])
-        if json_path:
-            self.bits_stats_to_json(json_path=json_path, bits_data=(max_bits_sum, total_bits_sum))
+    # def retrieve_bits_stats(self, bits_df, json_path=False):
+    #     max_bits_sum = sum([max(bits_df.iloc[ind]) for ind in bits_df.index])
+    #     total_bits_sum = sum([sum(bits_df.iloc[ind]) for ind in bits_df.index])
+    #     if json_path:
+    #         self.bits_stats_to_json(json_path=json_path, bits_data=(max_bits_sum, total_bits_sum))
 
-        return (max_bits_sum, total_bits_sum)
+    #     return (max_bits_sum, total_bits_sum)
 
-    def bits_stats_to_json(self, bits_data_json):
-        json_dict = {"Description": "The file is organized as a dict of protein IDs"
-                                    " (under results), with a list of (max_bits, total_bits)"
-                                    " from 1 to 1024 possible 5mers",
-                     "bits_data": {}}
+    # def bits_stats_to_json(self, bits_data_json):
+    #     json_dict = {"Description": "The file is organized as a dict of protein IDs"
+    #                                 " (under results), with a list of (max_bits, total_bits)"
+    #                                 " from 1 to 1024 possible 5mers",
+    #                  "bits_data": {}}
 
-        max_bits_list = []
-        json_dict["bits_data"][up_rrm_id][0].append(
-            logo.retrieve_bits_data(bits_df=bits_df)[0])
-        json_dict["bits_data"][up_rrm_id][1].append(
-            logo.retrieve_bits_data(bits_df=bits_df)[1])
+    #     max_bits_list = []
+    #     json_dict["bits_data"][up_rrm_id][0].append(
+    #         logo.retrieve_bits_data(bits_df=bits_df)[0])
+    #     json_dict["bits_data"][up_rrm_id][1].append(
+    #         logo.retrieve_bits_data(bits_df=bits_df)[1])
 
-    def generate_logo(self, bits_df, id):
+    def generate_logo_to_file(self, plot_path, top_scores, n_mers, window_size, dpi=300):
+        ppm = self.generate_PPM(rna_scores=top_scores, n_mers=n_mers, ws=window_size)
+        bits_df = self.ppm_to_bits(ppm=ppm)
+        self.generate_logo(bits_df=bits_df)
+        plt.savefig(plot_path, dpi=dpi)
+
+    def generate_logo(self, bits_df):
         bits_df = bits_df.astype('float64')
         # create Logo object
         ww_logo = logomaker.Logo(bits_df,
@@ -369,67 +390,63 @@ class LogoGenerator:
         # if os.path.exists(outfile_path) == False:
         #     plt.savefig(outfile_path, dpi=300)
 
-class BitsPlotter:
-    def __init__(self):
-        pass
+# class BitsPlotter:
 
-    def boxplot_bits_data_vs_kmers(self, x, y):
-        df = pd.DataFrame(list(zip(x, y)), columns=['n_kmers', 'bits_value'])
-        df['n_kmers_group'] = pd.cut(df['n_kmers'], bins=np.arange(min(x), max(x),
-                                                                   50))
-        f = plt.figure(figsize=(7, 4))
-        ax = f.add_subplot()
-        ax = sns.boxplot(x=df["n_kmers_group"], y=df["bits_value"],
-                         fliersize=2, color="lightgreen")
-        xlabels = np.arange(min(df["n_kmers"]), max(df["n_kmers"]), 50)[:-1]
-        xlabels = [i + 49 for i in xlabels]
+#     def boxplot_bits_data_vs_kmers(self, x, y):
+#         df = pd.DataFrame(list(zip(x, y)), columns=['n_kmers', 'bits_value'])
+#         df['n_kmers_group'] = pd.cut(df['n_kmers'], bins=np.arange(min(x), max(x),
+#                                                                    50))
+#         f = plt.figure(figsize=(7, 4))
+#         ax = f.add_subplot()
+#         ax = sns.boxplot(x=df["n_kmers_group"], y=df["bits_value"],
+#                          fliersize=2, color="lightgreen")
+#         xlabels = np.arange(min(df["n_kmers"]), max(df["n_kmers"]), 50)[:-1]
+#         xlabels = [i + 49 for i in xlabels]
 
-        ax.set_xticklabels(xlabels, fontsize=12, rotation=45)
-        ax.set_xlabel("Number of kmers used", fontsize=14)
-        ax.set_ylabel("Total bits", fontsize=12)
-        plt.tight_layout()
-        # plt.show()
-        plt.savefig("plots/total_bits_vs_n_kmers_boxplot_by_uniprot.png", dpi=300)
+#         ax.set_xticklabels(xlabels, fontsize=12, rotation=45)
+#         ax.set_xlabel("Number of kmers used", fontsize=14)
+#         ax.set_ylabel("Total bits", fontsize=12)
+#         plt.tight_layout()
+#         # plt.show()
+#         plt.savefig("plots/total_bits_vs_n_kmers_boxplot_by_uniprot.png", dpi=300)
 
-    def plot_score_range_vs_bits(self, json_dict):
-        range_score_bits_dict = json.load(open(json_dict))
+#     def plot_score_range_vs_bits(self, json_dict):
+#         range_score_bits_dict = json.load(open(json_dict))
 
-        n_mers_list = ["10", "50", "100", "250"]
+#         n_mers_list = ["10", "50", "100", "250"]
 
 
-        total_bits = [[sum([sum(n.values()) for n in [i for i in range_score_bits_dict[
-            up_id][nmers]["bits"].values()]]) for nmers in n_mers_list]
-             for up_id in range_score_bits_dict.keys()]
-        score_ranges = [[range_score_bits_dict[up_id]["all_scores"][0][1]- \
-              range_score_bits_dict[up_id]["all_scores"][int(nmers)][1] for
-              nmers in n_mers_list] for up_id in range_score_bits_dict.keys()]
+#         total_bits = [[sum([sum(n.values()) for n in [i for i in range_score_bits_dict[
+#             up_id][nmers]["bits"].values()]]) for nmers in n_mers_list]
+#              for up_id in range_score_bits_dict.keys()]
+#         score_ranges = [[range_score_bits_dict[up_id]["all_scores"][0][1]- \
+#               range_score_bits_dict[up_id]["all_scores"][int(nmers)][1] for
+#               nmers in n_mers_list] for up_id in range_score_bits_dict.keys()]
 
-        median_scores = []
-        for up_id in range_score_bits_dict.keys():
-            scores = [i[1] for i in range_score_bits_dict[up_id]["all_scores"]]
-            median_scores.append([np.median(scores[:int(nmers)]) for nmers in n_mers_list])
+#         median_scores = []
+#         for up_id in range_score_bits_dict.keys():
+#             scores = [i[1] for i in range_score_bits_dict[up_id]["all_scores"]]
+#             median_scores.append([np.median(scores[:int(nmers)]) for nmers in n_mers_list])
 
 
-        norm = matplotlib.colors.Normalize(vmin=-1, vmax=0.1)
-        cmap = cm.get_cmap('inferno')
-        for up_id, x, y, median in zip(range_score_bits_dict.keys(), total_bits,
-                                       score_ranges, median_scores):
-            plt.scatter(x, y, label=up_id, color=cmap(norm(median)), alpha=0.5)
-            # plt.plot(x, y, alpha=0.2, color="black", zorder=-10)
-
-        # Plot coloured by scores
-        sm = plt.cm.ScalarMappable(norm=norm, cmap=cmap)
-        plt.colorbar(sm, ax=None, orientation='vertical', label="Median score")
-
-        # Plot coloured by nmers
-        # colors = [cmap(norm(i)) for i in [0, 1, 2, 3]]
-        # lines = [plt.Line2D([0], [0], marker="o", color=c, linewidth=3, linestyle='-') for c in colors]
-        # labels = ['nmers=10', 'nmers=50', 'nmers=100', 'nmers=250']
-        # plt.legend(lines, labels)
-        plt.xlabel("Total bits", fontsize=12)
-        plt.ylabel("Score range", fontsize=12)
-        # plt.show()
-        plt.savefig("plots/total_bits_vs_score_range_per_UP_id_colored_by_score.png",
-                    dpi=300)
+#         norm = matplotlib.colors.Normalize(vmin=-1, vmax=0.1)
+#         cmap = cm.get_cmap('inferno')
+#         for up_id, x, y, median in zip(range_score_bits_dict.keys(), total_bits,
+#                                        score_ranges, median_scores):
+#             plt.scatter(x, y, label=up_id, color=cmap(norm(median)), alpha=0.5)
+#             # plt.plot(x, y, alpha=0.2, color="black", zorder=-10)
 
+#         # Plot coloured by scores
+#         sm = plt.cm.ScalarMappable(norm=norm, cmap=cmap)
+#         plt.colorbar(sm, ax=None, orientation='vertical', label="Median score")
 
+#         # Plot coloured by nmers
+#         # colors = [cmap(norm(i)) for i in [0, 1, 2, 3]]
+#         # lines = [plt.Line2D([0], [0], marker="o", color=c, linewidth=3, linestyle='-') for c in colors]
+#         # labels = ['nmers=10', 'nmers=50', 'nmers=100', 'nmers=250']
+#         # plt.legend(lines, labels)
+#         plt.xlabel("Total bits", fontsize=12)
+#         plt.ylabel("Score range", fontsize=12)
+#         # plt.show()
+#         plt.savefig("plots/total_bits_vs_score_range_per_UP_id_colored_by_score.png",
+#                     dpi=300)
